package com.miaxis.app.controller.film;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.miaxis.common.config.WxpayConfig;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.AesUtil;
import com.miaxis.feign.dto.FilmDTO;
import com.miaxis.feign.dto.FilmMcpData;
import com.miaxis.feign.dto.FilmXdResult;
import com.miaxis.film.domain.FilmOrder;
import com.miaxis.film.domain.RefundRecord;
import com.miaxis.film.dto.*;
import com.miaxis.film.service.IFilmOrderService;
import com.miaxis.film.service.IRefundRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.security.GeneralSecurityException;

import static com.miaxis.common.utils.OrderCodeFactory.getOrderCode;

@RestController
@RequiredArgsConstructor
@RequestMapping(Constants.OPEN_PREFIX+"/film/notify")
@Api(tags = {"【小程序-回调】"})
@Slf4j
public class NotifyController {


    @Autowired
    private WxpayConfig wxpayConfig;


    @Autowired
    private IFilmOrderService filmOrderService;

    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Autowired
    private  IRefundRecordService  refundRecordService;


    @Value("${film.notifyUrl}")
    private  String notifyUrl ;

    /**
     * 微信支付回调接口
     */
    @PostMapping(value = "/wxpay")
    @ApiOperation("微信支付回调")
    public WxNotifyReturnDTO wxpayNotify(@RequestBody FilmWxpayDTO filmWxpayDTO, HttpServletRequest request, BufferedReader br) throws GeneralSecurityException, IOException {
        String bodyString = getBodyString(br);
        System.out.println(bodyString);
        Boolean pass =  validate(request);

        String resourceString = getSourString(filmWxpayDTO);
        log.info(resourceString);
        JSONObject jsonObject = JSONObject.parseObject(resourceString);
        //将回调数据写入数据库
        writeNotifyDataToDb(jsonObject);

        WxNotifyReturnDTO wxNotifyReturnDTO = new WxNotifyReturnDTO();
        wxNotifyReturnDTO.setCode("SUCCESS");
        wxNotifyReturnDTO.setMessage("成功");
        return wxNotifyReturnDTO;
    }

    private String getBodyString(BufferedReader br) {
        String inputLine;

        String str = "";

        try {
            while ((inputLine = br.readLine()) != null) {
                str += inputLine;

            }

            br.close();

        } catch (IOException e) {
            System.out.println("IOException: " + e);

        }
        return str;
    }

    private Boolean validate(HttpServletRequest request) {
        String sign = request.getHeader("Wechatpay-Signature");
        String timestamp = request.getHeader("Wechatpay-Timestamp");
        String nonce = request.getHeader("Wechatpay-Nonce");


        StringBuffer sb = new StringBuffer();
        sb.append(timestamp + "\n");
        sb.append(nonce + "\n");
        System.out.println(sb);
        return true;

//        File file = new ClassPathResource("wechatpay/apiclient_key.pem").getFile();
//        String realPath =file.getAbsolutePath();
//        PrivateKey privateKey = CommonUtils.getPrivateKey(realPath);
//        // 进行签名服务
//        Signature signature = Signature.getInstance("SHA256withRSA");
//        signature.initSign(privateKey);
//        signature.update(sb.toString().getBytes("UTF-8"));
//        byte[] signedData = signature.sign();
//        String base64Str =  Base64.getEncoder().encodeToString(signedData);


    }


    /**
     * 微信退款回调
     */
    @PostMapping(value = "/refund")
    @ApiOperation("微信退款回调")
    public WxNotifyReturnDTO refundNotify(@RequestBody FilmWxpayDTO filmWxpayDTO, HttpServletRequest request) throws GeneralSecurityException, IOException {
        String resourceString = getSourString(filmWxpayDTO);
        log.info(resourceString);
        JSONObject jsonObject = JSONObject.parseObject(resourceString);
        //将回调数据写入数据库
        writeRefundNotifyDataToDb(jsonObject);

        WxNotifyReturnDTO wxNotifyReturnDTO = new WxNotifyReturnDTO();
        wxNotifyReturnDTO.setCode("SUCCESS");
        wxNotifyReturnDTO.setMessage("成功");

        return wxNotifyReturnDTO;
    }

    private void writeRefundNotifyDataToDb(JSONObject jsonObject) {
        String refundId = jsonObject.getString("refund_id");
        RefundRecord refundRecord = refundRecordService.getByRefundId(refundId);
        if (refundRecord == null) {
            throw new CustomException("该退款订单不存在");
        }
        refundRecord.setTransactionId(jsonObject.getString("transaction_id"));
        refundRecord.setUserReceivedAccount(jsonObject.getString("user_received_account"));
        refundRecord.setStatus(jsonObject.getString("refund_status"));
        refundRecordService.updateById(refundRecord);


    }

    /**
     * 电影回调接口
     */
    @PostMapping(value = "/filmOrder")
    @ApiOperation("电影订单回调")
    public FilmNotifyReturnDTO filmOrderNotify(FilmOrderNotifyDTO filmOrderNotifyDTO ,String sign, String appKey) {
        FilmOrder filmOrder = filmOrderService.getByOutTradeNo(filmOrderNotifyDTO.getThirdOrderId());
       //todo 检查sign
        String orderDataJson = filmOrder.getOrderDataJson();
        FilmOrderJsonData filmOrderJsonData = JSONObject.parseObject(orderDataJson, FilmOrderJsonData.class);
        FilmOrderNotifyDTO dbFilmOrderNotifyDTO= filmOrderJsonData.getFilmOrderNotifyDTO();
        if (dbFilmOrderNotifyDTO == null){
            dbFilmOrderNotifyDTO = filmOrderNotifyDTO;
        }else{
            dbFilmOrderNotifyDTO.setEventName(filmOrderNotifyDTO.getEventName());
            if (filmOrderNotifyDTO.getOrderPrice()!= null)
                dbFilmOrderNotifyDTO.setOrderPrice(filmOrderNotifyDTO.getOrderPrice());
            if (filmOrderNotifyDTO.getCloseCause()!= null)
                dbFilmOrderNotifyDTO.setCloseCause(filmOrderNotifyDTO.getCloseCause());
            if (filmOrderNotifyDTO.getRealSeat()!= null)
                dbFilmOrderNotifyDTO.setRealSeat(filmOrderNotifyDTO.getRealSeat());
            if (filmOrderNotifyDTO.getTicketCode()!= null)
                dbFilmOrderNotifyDTO.setTicketCode(filmOrderNotifyDTO.getTicketCode());
            if (filmOrderNotifyDTO.getTicketImage()!= null)
                dbFilmOrderNotifyDTO.setTicketImage(filmOrderNotifyDTO.getTicketImage());
        }
        filmOrderJsonData.setFilmOrderNotifyDTO(dbFilmOrderNotifyDTO);
        filmOrder.setOrderDataJson(JSONObject.toJSONString(filmOrderJsonData));
        //UserInfo student = SecurityUtils.getLoginUser().getStudent();
        //如果回调回来的订单关闭，则调用退款
        if ("ORDER_CLOSE".equals(filmOrderNotifyDTO.getEventName())){
            String refundCode = getOrderCode(null);
            filmOrder.setOutRefundNo(refundCode);
            threadPoolTaskExecutor.execute(new Runnable() {
            @Override
            public void run() {
                    try {

                        refundRecordService.refund(filmOrder,refundCode);
                    } catch (Exception e) {
                        log.info(e.getMessage());
                    }
                }
            });

        }

        filmOrderService.updateById(filmOrder);

        FilmNotifyReturnDTO filmNotifyReturnDTO = new FilmNotifyReturnDTO();
        filmNotifyReturnDTO.setCode("200");
        filmNotifyReturnDTO.setMessage("请求成功");
        filmNotifyReturnDTO.setSuccess(true);
        return filmNotifyReturnDTO;
    }


    /**
     * 电影下单测试
     */
    @PostMapping(value = "/filmOrder-test")
    @ApiOperation("电影订单下单测试")
    public Response<String> filmOrderNotifytest(String outTradeNo) throws GeneralSecurityException, IOException {
        String s = "";
        FilmOrder filmOrder1 = filmOrderService.getByOutTradeNo(outTradeNo);
        FilmDTO filmDTO = new FilmDTO();
        filmDTO.setUrl("api/order/create");
        StringBuffer paramData =  new StringBuffer();
        FilmOrderJsonData filmOrderJsonData = JSONObject.parseObject(filmOrder1.getOrderDataJson(), FilmOrderJsonData.class);
        FilmOrderCreateDTO filmOrder = filmOrderJsonData.getFilmOrderCreateDTO();
        paramData.append("showId="+filmOrder.getShowId()+"&");
        paramData.append("seat="+filmOrder.getSeat()+"&");
        if (filmOrder.getReservedPhone() != null){
            paramData.append("reservedPhone="+filmOrder.getReservedPhone()+"&");
        }
        paramData.append("thirdOrderId="+filmOrder.getThirdOrderId()+"&");
        paramData.append("notifyUrl="+notifyUrl+"&");
        if (filmOrder.getSeatId() != null){
            paramData.append("seatId="+filmOrder.getSeatId()+"&");
        }
        if (filmOrder.getSeatNo() != null){
            paramData.append("seatNo="+filmOrder.getSeatNo()+"&");
        }
        paramData.append("acceptChangeSeat="+filmOrder.getAcceptChangeSeat());
        filmDTO.setParamData(paramData.toString());
         s = filmOrderService.excuteFilmApi(filmDTO);
        log.info("电影下单返回值:"+s);
        return  Response.success(s);
    }




    @Transactional
    public void writeNotifyDataToDb(JSONObject jsonObject) {
        String outTradeNo = jsonObject.getString("out_trade_no");
        FilmOrder filmOrder = filmOrderService.getByOutTradeNo(outTradeNo);
        if (filmOrder == null) {
            throw new CustomException("该订单不存在");
        }
        filmOrder.setTransactionId(jsonObject.getString("transaction_id"));
        JSONObject amount = jsonObject.getJSONObject("amount");
        filmOrder.setPayerTotal(amount.getInteger("payer_total"));
        filmOrder.setTotal(amount.getInteger("total"));
        filmOrder.setCurrency(amount.getString("currency"));
        filmOrder.setPayerCurrency(amount.getString("payer_currency"));
        filmOrder.setTradeState(jsonObject.getString("trade_state"));
        filmOrder.setBankType(jsonObject.getString("bank_type"));
        DateTime dateTime  = new DateTime(jsonObject.getString("success_time"));
        filmOrder.setSuccessTime(dateTime.toDate());
        filmOrder.setTradeStateDesc(jsonObject.getString("trade_state_desc"));
        filmOrder.setTradeType(jsonObject.getString("trade_type"));
        filmOrder.setAttach(jsonObject.getString("attach"));
        JSONObject sceneInfo = jsonObject.getJSONObject("scene_info");
        if (sceneInfo != null){
            filmOrder.setDeviceId(sceneInfo.getString("device_id"));
        }
        //如果订单还未下过电影票订单，则异步调用电影下单api
        String  orderDataJson= filmOrder.getOrderDataJson();
        FilmOrderJsonData filmOrderJsonData = JSONObject.parseObject(orderDataJson, FilmOrderJsonData.class);
        if (filmOrderJsonData != null && filmOrderJsonData.getFilmOrderNotifyDTO() == null){
            threadPoolTaskExecutor.execute(new Runnable() {
                @Override
                public void run() {
                    FilmOrderCreateDTO filmOrderCreateDTO = filmOrderJsonData.getFilmOrderCreateDTO();
                    FilmDTO filmDTO = new FilmDTO();
                    if (filmOrderCreateDTO.getTicketType() ==1){
                        filmDTO.setUrl("api/order/create");
                    }else if (filmOrderCreateDTO.getTicketType() ==2){
                        filmDTO.setUrl("api/order/create-soon-order");
                    }

                    StringBuffer paramData =  new StringBuffer();
                    paramData.append("showId="+filmOrderCreateDTO.getShowId()+"&");
                    paramData.append("seat="+filmOrderCreateDTO.getSeat()+"&");
                    if (filmOrderCreateDTO.getReservedPhone() != null){
                        paramData.append("reservedPhone="+filmOrderCreateDTO.getReservedPhone()+"&");
                    }
                    paramData.append("thirdOrderId="+filmOrderCreateDTO.getThirdOrderId()+"&");
                    paramData.append("notifyUrl="+notifyUrl+"&");
                    if (filmOrderCreateDTO.getSeatId() != null){
                        paramData.append("seatId="+filmOrderCreateDTO.getSeatId()+"&");
                    }
                    if (filmOrderCreateDTO.getSeatNo() != null){
                        paramData.append("seatNo="+filmOrderCreateDTO.getSeatNo()+"&");
                    }
                    paramData.append("acceptChangeSeat="+filmOrderCreateDTO.getAcceptChangeSeat());
                    filmDTO.setParamData(paramData.toString());

                    String s = filmOrderService.excuteFilmApi(filmDTO);
                    log.info("电影下单返回值:"+s);
                    FilmXdResult filmXdResult = JSONObject.parseObject(s, FilmXdResult.class);
                    //如果返回值不是200 ，则调用退款流程
                    if (filmXdResult.getCode()!=200){
                        String refundCode = getOrderCode(null);
                        filmOrder.setOutRefundNo(refundCode);
                        try {
                            refundRecordService.refund(filmOrder,refundCode);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                    if (filmXdResult.getData()!= null){
                        FilmMcpData filmMcpData = filmXdResult.getData();
                        filmOrderJsonData.setFilmMcpData(filmMcpData);
                        filmOrder.setOrderDataJson(JSONObject.toJSONString(filmOrderJsonData));

                    }

                    filmOrderService.updateById(filmOrder);

                }
            });
        }


    }

    private String getSourString(FilmWxpayDTO filmWxpayDTO) throws GeneralSecurityException, IOException {
        AesUtil aesUtil = new AesUtil(wxpayConfig.getV3key().getBytes());
        FilmWxpayDTO.WxpaySource wxpaySource = filmWxpayDTO.getResource();
        return aesUtil.decryptToString(wxpaySource.getAssociated_data().getBytes(), wxpaySource.getNonce().getBytes(), wxpaySource.getCiphertext());
    }




    @Data
    public class FilmNotifyReturnDTO {
        String code;
        String message;
        Boolean success;
    }

}
